/*
	*==============================*
	|      The PackTracker v1.2    |
	|  by Henrik 'TwoD' Danielsson |
	| the_dragonwizard@hotmail.com |
	*==============================*
	This script allows you to see the status of your deliveries as long as you have been provided with a Kolli Id for tracking purposes.
	The script is designed to download an XML file generated by the free package tracking service at www.posten.se.
	The XML file is then stored in your Samurize scripts folder and used as a cache, to avoid having to dowload the file
	once for each value you want to monitor.

	The Kolli Id is a long number identifying your package(s).
	This needs to be entered into the KolliId variable below. Do not include the two letter language code!

	To track multiple packages, simply make one copy of the script for each package.

	Note that this script is only useable for the Swedish Post!
	If you'd like a similar program for a foreign postal service, I'd be happy to help.
	
	You are free to use this script as you wish, but please keep the box at the top if extending the script.
	It would also be great if you sent me a line or two about the updates you've made, that way I can improve
	this script further.
	
	Note that the tracking system sometimes does not enter a time/date stamp in the xml. This is NOT a problem with the script.
	
	Changelog:
	v1.2
	* The timer was corrected. Multiplied once too much by 60...

	v1.1
	* The script now sends the language code properly. This caused problems with some kolli numbers.
	* The XML file now has the kolli number appended to the filename, this is to avoid mixup problems between different packages.
	* The script now aborts correctly if there is an error when looking up the Samurize folder.

*/


var KolliId="89235610243"; // This is the Kolli Id you need to change for each package you want to track.
var Unknown="Oknd"; // This text is shown when a field could not be read
var Weekdays=["Sndag","Mndag","Tisdag","Onsdag","Torsdag","Fredag","Lrdag"];
var Months=["Januari","Februari","Mars","April","Maj","Juni","Juli","Augusti","September","Oktber","November","December"];
var PageError="Error vid nedladdning"; // This text is shown when there was an error when downloading the XML file.
var UpdateInterval = 20; // Minutes between xml file reloads (Note that this value won't be exact, depending on the "Run every X min" value in Samurize
/*
	These functions serve as interface for Samurize
*/
function getKolliId(){return KolliId;}
function getStatusDescription(){return getField_("statusdescription");}
function getServiceName(){return getField_("servicename");}
function getReceiverZipcode(){return getField_("receiverzipcode");}
function getCustomerName(){return getField_("customername");}
function getCustomerNo(){return getField_("customerno");}
function getReceiverCity(){return getField_("receivercity");}
function getActualWeight(){return getField_("actualweight");}
function getSentDate(){return getField_("datesent");}
function getSentDay(){return getField_("datesent").substring(6);}
function getSentMonth(){return getField_("datesent").substring(4,6);}
function getSentYear(){return getField_("datesent").substring(0,4);}
function getSentWeekDay(){return Weekdays[new Date(parseInt(getSentYear(),10),parseInt(getSentMonth())-1,parseInt(getSentDay()),10).getDay()];}
function getSentMonthName(){return Months[parseInt(parseInt(getSentMonth()),10)-1];}
function getDeliveredDate(){return getField_("datedelivered");}
function getDeliveredDay(){return getField_("datedelivered").substring(6);}
function getDeliveredMonth(){return getField_("datedelivered").substring(4,6);}
function getDeliveredYear(){return getField_("datedelivered").substring(0,4);}
function getWeekDayDelivered(){return Weekdays[new Date(parseInt(getDeliveredYear(),10),parseInt(getDeliveredMonth(),10)-1,parseInt(getDeliveredDay(),10)).getDay()];}
function getMonthNameDelivered(){return Months[parseInt(getDeliveredMonth(),10)-1];}
function getLastEventLocation(){return getLastEventField_("location");}
function getLastEventTime(){var time=getLastEventField_("time");return time.substring(0,2)+":"+time.substring(2);}
function getLastEventDate(){return getLastEventField_("date");}
function getLastEventDay(){return getLastEventField_("date").substring(6);}
function getLastEventMonth(){return getLastEventField_("date").substring(4,6);}
function getLastEventYear(){return getLastEventField_("date").substring(0,4);}
function getLastEventDescription(){return getLastEventField_("description");}
function getLastEventWeekDay(){return Weekdays[new Date(parseInt(getLastEventYear(),10),parseInt(getLastEventMonth(),10)-1,parseInt(getLastEventDay(),10)).getDay()];}
function getLastEventMonthName(){return Months[parseInt(getLastEventMonth(),10)-1];}
function ForceXmlDownload(){return getPage_(true)?"Done":PageError;} // Not recommended to run often!

/*
The functions below are not directly callable by Samurize
*/

function getPage_(force){
	var fileFound=false;
	try{
		var samurizeDir = new ActiveXObject("WScript.Shell").RegRead("HKEY_CURRENT_USER\\Software\\Serious Samurize\\General\\DirPath");
		var filepath = samurizeDir + "Scripts\\" + "packTracked_"+KolliId+".xml";
	}
	catch(er){
		return false // Error when accessing the registry to find the Samurize folder.
	}
	try{
		var fs = new ActiveXObject("Scripting.FileSystemObject");
		if(fs.FileExists(filepath)){ // Check If XML File Is Cached
			var file=fs.GetFile(filepath);
			var filetime = new Date(file.DateLastModified).getTime();
			var now = new Date().getTime();
			if(now-filetime<60000*UpdateInterval){ // The downloaded file is less UpdateInterval minutes old.
				xmlDoc = new ActiveXObject("Msxml2.DOMDocument");
				xmlDoc.async = false;
				xmlDoc.resolveExternals = false;
				xmlDoc.load(filepath);
				Xml=xmlDoc;
				fileFound=true;
			}
		}
	}
	catch(er){fileFound=false;}

	if(!fileFound || force){
		try{ // Get XML File
			var http = new ActiveXObject("Microsoft.XMLHTTP");
			http.Open("GET","http://server.logistik.posten.se/servlet/PacTrack?lang=SE&kolliid="+KolliId+"&"+Math.random(),false);
			http.send();
			Xml=http.responseXml;
		}
		catch(e){return false;}

		try{ // Cache XML File
			var fs = new ActiveXObject("Scripting.FileSystemObject");
			filepath = fs.CreateTextFile(filepath,true,true); // Overwrite and use Unicode
			filepath.Write(http.responseXml.xml);
			filepath.Close();
		}
		catch(er){/*Error when saving xml file. It must be downloaded once for every value read.*/}
	}
	return Xml;
}

function getField_(what){
	var Xml=false;
	if(!(Xml=getPage_(false))){return PageError;}
	try{
		return Xml.getElementsByTagName("parcel").item(0).getElementsByTagName(what).item(0).firstChild.nodeValue;
	}
	catch(er){return Unknown;}
}

function getLastEventField_(what){
	var Xml=false;
	if(!(Xml=getPage_(false))){return PageError;}
	try{
		var events=Xml.getElementsByTagName("parcel").item(0).getElementsByTagName("event");
		var lastEvent=events.length>0?events.item(events.length-1):false;
		var errorEvents=Xml.getElementsByTagName("parcel").item(0).getElementsByTagName("errorevent");
		var lastErrorEvent=errorEvents.length>0?errorEvents.item(errorEvents.length-1):false;
		var theEvent=lastErrorEvent && parseInt(lastErrorEvent.getElementsByTagName("date").item(0).firstChild.nodeValue,10)>=parseInt(lastEvent.getElementsByTagName("date").item(0).firstChild.nodeValue,10) && parseInt(lastErrorEvent.getElementsByTagName("time").item(0).firstChild.nodeValue,10)>=parseInt(lastEvent.getElementsByTagName("time").item(0).firstChild.nodeValue,10)?lastErrorEvent: (lastEvent?lastEvent:false);
		return theEvent?theEvent.getElementsByTagName(what).item(0).firstChild.nodeValue:Unknown;
	}
	catch(er){return Unknown;}
}